function recon = reconstructInput(rbm, data, binary_vis, numgibbs, T)
%given a trained RBM, reconstruct the given data with gibbs sampling and
%plot the data and reconstructions
% rbm = {weights, hidden biases, visible biases};
% data = [width, height, num_samples];
% binary_vis = if true visible units are sampled to binary values,
%       otherwise activation probabilities are treated as grayscale values.
% numgibbs = the number of reconstructions to compute and plot.
% T = temperature (0,inf).  The higher the temperature, the more stochastic
%       the reconstructions  (e.g. 1/4, 1/2, 1, 2)
if nargin<4; fprintf('Usage: reconstructInput(rbm, data, binary, nsteps, T)\n'); return; end
[width,height,nsamples]=size(data);  [w,hb,vb] = rbm{:};  [nh,nv]=size(w); 
recon = reshape(data, width*height, nsamples);        %put data in matrix form
if nsamples>10; nsamples=10; recon=recon(:,1:10); end %only plot first 10
figure; set(gcf,'Position',get(0,'screensize'));      %maximize figure window

for i=0:numgibbs
  for j=1:nsamples
    subplot(numgibbs+1, nsamples, i*nsamples + j);
    imagesc( reshape(recon(:,j),[width height]) );  %plot reconstruction
    colormap gray; axis off; axis image;
    
    h = 1./(1 + exp(-1/T.*(w *recon(:,j) + hb)));   %compute hidden probs
    hs= h > rand(nh,1);                             %sample hidden probs
    recon(:,j) = 1./(1 + exp(-1/T.*(w'*hs + vb)));  %compute visible probs
    if binary_vis; recon(:,j) = recon(:,j) > rand(nv,1); end %sample vis
  end
  drawnow;
end