function [rbm,output] = trainrbm(input, nh, epochs, e, m)
%Given a series of inputs, trains an RBM with nh hidden units.
%Plotted error is the average across visible units and inputs.
% rbm{1}= weights (w), rbm{2}= hidden biases (hb), rbm{3}= visible biases (vb)
% output= hidden activation probabilities for every input (after training)
% input = [dimesion, num_samples]
% nh    = number of hidden units
% e     = learning rate (default 0.01)
% m     = momentum      (default 0).   Range [0,1-e]     (e.g. .9).

[nv,nsamples]=size(input);   output = zeros(nh,nsamples);
vb =zeros(nv,1);  hb =zeros(nh,1);  w =.1*randn(nh,nv);
dvb=zeros(nv,1);  dhb=zeros(nh,1);  dw=   zeros(nh,nv);  %d for delta

figure; hold on; xlabel epoch; ylabel 'avg error'; xlim([0 epochs]);
for epoch=1:epochs;
  err=0;  %fprintf(1, 'epoch %3i ', epoch);
  for i=1:nsamples
    %Sample hidden units given input, then reconstruct visible and hidden.
    v = input(:,i);
    h = 1./(1 + exp(-(w *v + hb)));   %sigmoid activation function
    hs= h > rand(nh,1);               %probabilistic sampling of binary states
    vr= 1./(1 + exp(-(w'*hs+ vb)));   %input reconstruction
    hr= 1./(1 + exp(-(w *vr+ hb)));   %hidden reconstruction

    %Contrastive Divergence rule: dw ~ h*v - hr*vr
    dw  = m*dw  + e*(h*v'-hr*vr');  w = w +dw;
    dvb = m*dvb + e*( v  -  vr  );  vb= vb+dvb;
    dhb = m*dhb + e*( h  -  hr  );  hb= hb+dhb;
    
    err = err   + sum( (v-vr).^2 );   %reconstruction error
    output(:,i)=h;
  end
 %fprintf(1, ' error %.4f  \n', err/(nv*nsamples)); %text output
  plot( epoch, err/(nv*nsamples), '.');  drawnow;   %figure output
end
rbm = {w,hb,vb}; %alternatively, rbm = struct('w',w,'hb',hb,'vb',vb);